#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

extern byte[] SpriteChars;

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + VramRowSize * 1 + 32, "SCORE");
    PrintS(Vram + VramRowSize * 5 + 32, "HI-SCORE");
    PrintS(Vram + VramRowSize * 9 + 32, "STAGE");
    PrintByteNumber2(Vram + VramRowSize * 9 + 38, CurrentStage + 1);
    PrintS(Vram + VramRowSize * 12 + 32, "TIME");

    {
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 21 + 33;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            do {
                ptr<byte> pChar;
                pChar = SpriteChars + Char_Man;
                repeat (2) {
                    repeat (2) {
                        *pVram = *pChar; ++pVram;
                        ++pChar;
                    }
                    pVram += VramRowSize - 2;
                }
                pVram += 3 - VramRowSize * 2;
                --i;
            } while (i > 0);
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    WaitVBlank();

    pVram = PrintNumber5(Vram + VramRowSize * 2 + 34, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize * 6 + 34, HiScore);
    PrintC(pVram, '0');
}

void PrintTime()
{
    WaitVBlank();
    PrintByteNumber3(Vram + VramRowSize * 12 + 37, StageTime);
}

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramRowSize * 10 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 11 + 12, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 11 + 13, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
		0xa0, 0xe2, 0xfb, 0x7b, 
		0xa0, 0x62, 0xfe, 0x7e, 
		0xa0, 0x20, 0xe1, 0x61, 
		0xe2, 0xe2, 0xe2, 0x20, 
		0x20, 0x20, 0x20, 0x20, 
		0xfe, 0xfb, 0x7b, 0xfe, 
		0xa0, 0xe1, 0x61, 0xa0, 
		0x7c, 0xe2, 0x20, 0x7c, 
		0x20, 0x20, 0x6c, 0x7b, 
		0xfb, 0x7b, 0xfb, 0xec, 
		0xe1, 0x61, 0xe1, 0x61, 
		0xe2, 0x20, 0x20, 0xe2, 
		0x20, 0x20, 0x20, 0xe1, 
		0x6c, 0xec, 0xe2, 0xe1, 
		0x20, 0xe2, 0xfc, 0xe1, 
		0x7c, 0xe2, 0x7e, 0x7c, 
		0x61, 0x20, 0x20, 0x20, 
		0xfc, 0xec, 0x6c, 0xec, 
		0xa0, 0x7b, 0xe1, 0xec, 
		0x7e, 0xe2, 0x20, 0xe2, 
		0x20, 0xe1, 0x61, 0xa0, 
		0xfc, 0xe1, 0x61, 0xa0, 
		0xe2, 0xe1, 0x61, 0xa0, 
		0x7e, 0x7c, 0x7e, 0xe2, 
    };

    ClearScreen(); 
    HideAllSprites();
    PrintStatus();
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 8 + 4;
        p = TitleBytes;
        repeat (6) {
            repeat (4) {
                repeat (4) {
                    *pVram = *p; ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 20 + 9, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 21 + 9, "OR SHIFT KEY");
}


void DrawFence()
{
    ptr<byte> pTop, pBottom;
    pTop = Vram;
    pBottom = Vram + VramRowSize * 23;
    repeat(StageWidth) {
        *pTop = Char_FenceTop; ++pTop;
        *pBottom = Char_FenceBottom; ++pBottom;
    }
}